/* Copyright (c) 2012 Andy McDonald. All rights reserved. */
/* Please refer to licence.txt for licensing terms. */

// ====================================================================================
var EXPORTED_SYMBOLS = ['FoxClocks_Engine'];

// ====================================================================================
(function() {
	"use strict";

	// ====================================================================================
	const CI = Components.interfaces, CC = Components.classes, CU = Components.utils;
	CU.import("resource://foxclocks/logger.js");
	CU.import("resource://foxclocks/utils.js");
	CU.import("resource://foxclocks/updatemanager.js");
	CU.import("resource://foxclocks/prefmanager.js");
	CU.import("resource://foxclocks/watchlistmanager.js");
	CU.import("resource://foxclocks/zonemanager.js");

	const _WAIT_FOR_SESSION_RESTORED = false; // AFM - for now
	
	// ====================================================================================
	function Engine()
	{
		FoxClocks_Logger.init(FoxClocks_PrefManager.getPref("foxclocks.logging.level"), FoxClocks_PrefManager.getPref("foxclocks.logging.tmpfile.enabled"));

		this._started = null; // AFM - tri-state boolean
		this._observerService = CC["@mozilla.org/observer-service;1"].getService(CI.nsIObserverService);
		
		var appName = FoxClocks_Utils.getAppInfo().appName;
		if (_WAIT_FOR_SESSION_RESTORED && (appName === 'Firefox' || appName === 'SeaMonkey')) // AFM - event only for these apps
		{
			this._sessionRestored = false;

			var self = this;
			var restoreObserver = { observe: function(subject, topic, data) {
				self._observerService.removeObserver(restoreObserver, "sessionstore-windows-restored");
				
				self._sessionRestored = true;
				if (self._started === true)
					self._checkVersion();
			}};
		
			// AFM - ASAP or we might miss the event
			//
			this._observerService.addObserver(restoreObserver, "sessionstore-windows-restored", false);
		}
		else
		{
			FoxClocks_Logger.log("Engine::Engine(): not waiting for session restore");
			this._sessionRestored = true
		}

		FoxClocks_PrefManager.declarePrefAsXml("foxclocks.watchlist");
	};
	
	// ====================================================================================
	Engine.prototype =
	{
		start: function()
		{
			if (this._started !== null)
			{
				FoxClocks_Logger.log("Engine::start(): startup already in progress");
				return;
			}
			
			this._started = false;
			FoxClocks_Logger.log("Engine::start(): starting up...");
			
			var self = this;
			FoxClocks_ZoneManager.loadZoneData(function(success) {

				if (success === false)
				{
					self.shutdown();
					FoxClocks_Logger.fatal("Engine::start(): startup aborted");
					return;
				}
				
				self._initUpdateManager();
								
				FoxClocks_ZoneManager.loadZonePicker(function(success) {

					if (success === false)
					{
						self.shutdown();
						FoxClocks_Logger.fatal("Engine::start(): startup aborted");
						return;
					}
					
					// AFM - empty watchlist names are taken from the zone picker, so we need to wait for the zone picker
					// to be available before we create the watchlist
					//
					FoxClocks_WatchlistManager.watchlistFromXmlString(FoxClocks_ZoneManager.getZoneData(), FoxClocks_PrefManager.getPref("foxclocks.watchlist"));

					self._started = true;
					self._observerService.notifyObservers(self, "foxclocks", "engine:started");
					FoxClocks_Logger.log("Engine::start(): startup done");

			 		FoxClocks_PrefManager.addPrefObserver("foxclocks.", self);
					FoxClocks_PrefManager.addPrefObserver("extensions." + FoxClocks_Utils.FC_GUID_FOXCLOCKS + ".", self);
					self._observerService.addObserver(self, "foxclocks", false);

					if (self._sessionRestored)
						self._checkVersion();	
				});
			});
		},

		// ====================================================================================
		onStarted: function(callback)
		{
			if (this._started === true)
			{
				callback();
			}
			else
			{
				var observerService = this._observerService;
				var startupObserver = { observe: function(subject, topic, data) {
					if (data === 'engine:started')
					{
						observerService.removeObserver(startupObserver, "foxclocks");
						callback();
					}
				}};
				
				observerService.addObserver(startupObserver, "foxclocks", false);
			}
		},
		
		// ====================================================================================
		shutdown: function()
		{
			FoxClocks_PrefManager.removePrefObserver("foxclocks.", this);
			FoxClocks_PrefManager.removePrefObserver("extensions." + FoxClocks_Utils.FC_GUID_FOXCLOCKS + ".", this);
			try {this._observerService.removeObserver(this, "foxclocks");} catch(ex){}
			try {this._observerService.removeObserver(this, "sessionstore-windows-restored");} catch(ex){}

			this._started = null;
			// AFM - don't reset this._sessionRestored = false;
		},

		// ====================================================================================
		// AFM - nsIObserver
		observe: function(aSubject, aTopic, aData)
		{					
			if (aTopic == "foxclocks")
			{				
				if (aData == "updatemanager:update-complete")
				{
					this._onZoneDataUpdate();
				}
			}
			else if (aTopic == "nsPref:changed")
			{
				switch (aData)
				{
					case "foxclocks.logging.level": FoxClocks_Logger.init(FoxClocks_PrefManager.getPref("foxclocks.logging.level"), null); break;
					case "foxclocks.logging.tmpfile.enabled": FoxClocks_Logger.init(null, FoxClocks_PrefManager.getPref("foxclocks.logging.tmpfile.enabled")); break;
					case "foxclocks.data.update.auto.enabled": this._initUpdateManager(); break;
					case "foxclocks.data.update.noserver.updateurl": this._initUpdateManager(); break;
					case "foxclocks.watchlist":
					{
						FoxClocks_WatchlistManager.watchlistFromXmlString(FoxClocks_ZoneManager.getZoneData(), FoxClocks_PrefManager.getPref("foxclocks.watchlist"));
						this._observerService.notifyObservers(FoxClocks_WatchlistManager, "foxclocks", "engine:watchlist-changed");
						break;	
					}
							
					case "foxclocks.zonepicker.dataurl":
					{
						var self = this;
						FoxClocks_ZoneManager.loadZonePicker(function(success) {
							if (success)
								self._observerService.notifyObservers(null, "foxclocks", "engine:zone-picker-changed");
						}, true);

						break;
					}	
				}
			}
			else if (aTopic == "alertclickcallback")
			{
				if (aData == "engine-internal:foxclocks-zone-data-update-complete:new")
					FoxClocks_Utils.openFoxClocksDbUpdate();
			}
		},

		// ====================================================================================
		_checkVersion : function()
		{
			FoxClocks_Utils.getFoxClocksVersion(function(foxClocksVersion) {
				
				FoxClocks_Logger.log("Engine::_checkVersion(): version " + foxClocksVersion);
				
				var prevRunVersion = FoxClocks_PrefManager.getPref("extensions." + FoxClocks_Utils.FC_GUID_FOXCLOCKS + ".prevrun.version");				
				FoxClocks_PrefManager.setPref("extensions." + FoxClocks_Utils.FC_GUID_FOXCLOCKS + ".prevrun.version", foxClocksVersion);
				
				if (prevRunVersion == "")
				{
					FoxClocks_Utils.openFoxClocksInstall(foxClocksVersion);
				}
				else
				{
					var comparison = Components.classes["@mozilla.org/xpcom/version-comparator;1"]
						.getService(Components.interfaces.nsIVersionComparator).compare(prevRunVersion, foxClocksVersion);
		
					if (comparison < 0)
						FoxClocks_Utils.openFoxClocksUpdate(foxClocksVersion, prevRunVersion);
				}
			});
		},
	
		// ====================================================================================
		_initUpdateManager: function()
		{
			FoxClocks_Logger.log("+Engine::_initUpdateManager()");
			
			var autoUpenabled = FoxClocks_PrefManager.getPref("foxclocks.data.update.auto.enabled");	

			var noServerUrl = FoxClocks_PrefManager.getPref("foxclocks.data.update.noserver.updateurl");
			var remoteUrlIsRawData = noServerUrl != null && noServerUrl != "";

			if (remoteUrlIsRawData)
			{
				var remoteUrl = noServerUrl;
			}
			else
			{
				var remoteUrl = encodeURI(FoxClocks_Utils.FC_URL_DATABASE_UPDATE_CHECK +
					"?client=" + FoxClocks_Utils.getAppInfo().appName +
					"&data_source_version=" + FoxClocks_ZoneManager.dataSource.version);
			}
				
			FoxClocks_UpdateManager.init(autoUpenabled, remoteUrl, remoteUrlIsRawData);
			
			// AFM - the way the GUI is notified of these changes is poor; if the devel param is modified,
			// eg, the GUI doesn't update (because it's not watching the extensions.{ branch
			//
			
			FoxClocks_Logger.log("-Engine::_initUpdateManager()");
		},
		
		// ====================================================================================		
		_onZoneDataUpdate: function()
		{
			if (FoxClocks_UpdateManager.getLastUpdateResult().result != "OK_NEW")
			{
				this._observerService.notifyObservers(FoxClocks_UpdateManager, "foxclocks", "engine:zone-data-update-complete");				
				return;	
			}
	              
			var self = this;
			FoxClocks_ZoneManager.loadZoneData(function(success) {

				if (success === true)
				{
					FoxClocks_Logger.log("Engine::_onZoneDataUpdate(): reinitialised on new data");
				
					if (FoxClocks_PrefManager.getPref("foxclocks.data.update.auto.alert.enabled") == true)
					{
						var alertText =  CC["@mozilla.org/intl/stringbundle;1"].getService(CI.nsIStringBundleService)
							.createBundle("chrome://foxclocks/locale/foxclocks.properties")
							.GetStringFromName("options.data.update.last.status.ok_new.label");
						
						try
						{
							// AFM  observe 'alertclickcallback', 'alertfinished'
							//
							CC["@mozilla.org/alerts-service;1"].getService(CI.nsIAlertsService).showAlertNotification("chrome://foxclocks/skin/icon.png",
									"FoxClocks", alertText, true, "engine-internal:foxclocks-zone-data-update-complete:new", self, "FoxClocks");
						}
						catch (ex)
						{
							FoxClocks_Logger.error("Engine::_onZoneDataUpdate(): " + ex); // AFM - NS_ERROR_NOT_AVAILABLE is a known exception
						}
					}
				}
				else
				{
					FoxClocks_Logger.error("Engine::_onZoneDataUpdate(): could not reinitialise on new data");
				}
				
				self._observerService.notifyObservers(FoxClocks_UpdateManager, "foxclocks", "engine:zone-data-update-complete");
			});	
		}
	};

	// ====================================================================================
	var _engine = new Engine();

	this.FoxClocks_Engine = {
		start: function() { _engine.start(); },
		onStarted: function(callback) { _engine.onStarted(callback); }
	};

}).call(this);