/* Copyright (c) 2012 Andy McDonald. All rights reserved. */
/* Please refer to licence.txt for licensing terms. */

// ====================================================================================
var EXPORTED_SYMBOLS = ["FoxClocks_PrefManager"];

// ====================================================================================
(function() {
	"use strict";

	// ====================================================================================
	const CI = Components.interfaces, CC = Components.classes, CU = Components.utils;
	CU.import("resource://foxclocks/logger.js");
	CU.import("resource://foxclocks/utils.js");

	//====================================================================================
	var _prefService = CC["@mozilla.org/preferences-service;1"].getService(CI.nsIPrefBranch);
	
	// ====================================================================================
	function PrefManager()
	{
		this._prefsDeclaredAsXml = [];
	}

	// ====================================================================================
	PrefManager.prototype =
	{
		// ====================================================================================
		getPrefTypeString : function(prefType)
		{
			if (prefType == CI.nsIPrefBranch.PREF_STRING)
				return "PREF_STRING";
			else if (prefType == CI.nsIPrefBranch.PREF_INT)
				return "PREF_INT";
			else if (prefType == CI.nsIPrefBranch.PREF_BOOL)
				return "PREF_BOOL";
			else
				return "PREF_INVALID";
		},		
		
		// ====================================================================================
		getPref : function(prefName)
		{
			try
			{
				var prefType = _prefService.getPrefType(prefName);
				
				// AFM - don't use getCharPref(): all string prefs are unicode
				//
				if (prefType == _prefService.PREF_STRING)
				{		
					var data = _prefService.getComplexValue(prefName, CI.nsISupportsString).data;
					
					// AFM - there's no way to know whether a pref has a localised default, but this works well
					// unless there's really a pref which is a chrome URL
					//
					if (data.substring(0, 9) == "chrome://")
						data = _prefService.getComplexValue(prefName, CI.nsIPrefLocalizedString).data;
		
					return data;
				}	
				else if (prefType == _prefService.PREF_INT)
				{
					return _prefService.getIntPref(prefName);
				}
				else if (prefType == _prefService.PREF_BOOL)
				{
					return _prefService.getBoolPref(prefName);
				}
				else
				{
					return null;
				}
			}
			catch (ex)
			{	
				FoxClocks_Logger.error("PrefManager::getPref(): error on pref <" + prefName + ">: " + ex);
				return null;
			}
		},
		  
		// ====================================================================================
		setPref : function(prefName, value)
		{
			var prefType = _prefService.getPrefType(prefName);
			
			if (prefType == _prefService.PREF_STRING)
			{
				// AFM - don't use setCharPref(): all string prefs are unicode
				//
				var supportsString = CC["@mozilla.org/supports-string;1"].createInstance(CI.nsISupportsString);
				supportsString.data = value;
			
				_prefService.setComplexValue(prefName, CI.nsISupportsString, supportsString);
			}
			else if (prefType == _prefService.PREF_INT)
			{
				_prefService.setIntPref(prefName, value);
			}
			else if (prefType == _prefService.PREF_BOOL)
			{
				_prefService.setBoolPref(prefName, value);
			}
		},
		
		// ====================================================================================
		addPrefObserver : function(branchName, observer)
		{
			var nsIPrefBranchInternal = _prefService.QueryInterface(CI.nsIPrefBranchInternal);
			nsIPrefBranchInternal.addObserver(branchName, observer, false);
		},
		
		// ====================================================================================
		removePrefObserver : function(branchName, observer)
		{
			var nsIPrefBranchInternal = _prefService.QueryInterface(CI.nsIPrefBranchInternal);
			nsIPrefBranchInternal.removeObserver(branchName, observer);
		},
	
		// ====================================================================================
		declarePrefAsXml : function(fqPrefName, truth)
		{
			// AFM - the pref must not be a document fragment
			//
			this._prefsDeclaredAsXml[fqPrefName] = (typeof(truth) == "undefined" || truth == true); 
		},
		
		// ====================================================================================
		prefIsDeclaredAsXml : function(fqPrefName)
		{
			var truth = this._prefsDeclaredAsXml[fqPrefName];
			return (typeof(truth) != "undefined" && truth == true); 
		},
	
		// ====================================================================================
		prefsToXml : function(branchName, xmlns, rootElt, version)
		{    
			var domParser = CC["@mozilla.org/xmlextras/domparser;1"].createInstance(CI.nsIDOMParser);
	
			var doc = FoxClocks_Utils.getDOMImpl().createDocument(xmlns, rootElt, null);
			var root = doc.documentElement;
			
			if (version != null)
				root.setAttribute("version", version);
				
			var prefsList = _prefService.getChildList(branchName, {});
		        
			for (var i=0; i < prefsList.length; i++)
			{
				var currPref = prefsList[i];
				var currPrefTypeStr = this.getPrefTypeString(_prefService.getPrefType(currPref));
							
				var elt = doc.createElementNS(xmlns, "Pref");
				root.appendChild(elt);
				elt.setAttribute("id", currPref);
				elt.setAttribute("type", currPrefTypeStr);
				
				var currPrefValue = this.getPref(currPref);
				
				if (this.prefIsDeclaredAsXml(currPref))
				{
					var valueNode = doc.createElementNS(xmlns, "Value");
					elt.appendChild(valueNode);
	
					// AFM - the pref must not be a document fragment
					//
					var currPrefValueAsDoc = domParser.parseFromString(currPrefValue, "text/xml");
					var nodeToImport = doc.importNode(currPrefValueAsDoc.documentElement, true); // true => deep copy
					valueNode.appendChild(nodeToImport);
				}
				else
				{
					elt.setAttribute("value", currPrefValue);
				}
			}
			
			return doc; 
		},
		
		// ====================================================================================
		xmlToPrefs : function(branchName, doc)
		{
			var serializer = CC["@mozilla.org/xmlextras/xmlserializer;1"].createInstance(CI.nsIDOMSerializer);
			
			var root = doc.documentElement;
											
			if (root.nodeName == "parsererror")
				return false;
		
			// AFM - hack: beta versions of FoxClocks 2.0 went out with root node
			// <FoxClocks>. These are not supported
			//
			if (root.nodeName == "FoxClocks")
			{
				FoxClocks_Logger.error("PrefManager::xmlToPrefs(): unsupported format");
				return false;
			}
				
			var importedPrefs = [];			
			var prefNodes = root.getElementsByTagName("Pref");
	
			for (var i=0; i < prefNodes.length; i++)
			{
				var prefNode = prefNodes.item(i);
				prefNode.QueryInterface(CI.nsIDOMElement);
				
				var id = prefNode.getAttribute("id");
				var value = prefNode.getAttribute("value");
				var type = prefNode.getAttribute("type");
				
				if (id == null || type == null)
				{
					FoxClocks_Logger.warn("PrefManager::xmlToPrefs(): pref node has missing attributes - skipping");
				}
				else
				{
					if (importedPrefs[id] != null)
						FoxClocks_Logger.warn("PrefManager::xmlToPrefs(): duplicate pref <" + id + "> - overwriting");
					
					// AFM - pref is stored in the 'value' attribute
					//
					if (value != null)
					{	
						importedPrefs[id] = (type == "PREF_BOOL") ? (value == "true") : value;
					}
					else
					{
						var firstValueChildNode = null;
						
						var valueNode = FoxClocks_Utils.getFirstEltByTagAsNode(prefNode, "Value");
						if (valueNode != null)
							firstValueChildNode = FoxClocks_Utils.getFirstEltByTagAsNode(valueNode, "*");
						
						if (firstValueChildNode != null)
							importedPrefs[id] = serializer.serializeToString(firstValueChildNode);
						else
							FoxClocks_Logger.warn("PrefManager::xmlToPrefs(): xml pref <" + id + "> has missing value - skipping");
					}	
				}
			}		
				
			var prefsList = _prefService.getChildList(branchName, {});
			for (var i=0; i < prefsList.length; i++)
			{
				var currPref = prefsList[i];
				var value = importedPrefs[currPref];
				
				if (value == null)
				{
					FoxClocks_Logger.warn("PrefManager::xmlToPrefs(): document does not contain pref <" + currPref + ">");
				}		
				else if (value == this.getPref(currPref))
				{
					FoxClocks_Logger.log("PrefManager::xmlToPrefs(): pref <" + currPref + ">, value <" + value + "> - no change");
				}
				else
				{
					FoxClocks_Logger.log("PrefManager::xmlToPrefs(): pref <" + currPref + ">, value <" + value + "> - changed");
					this.setPref(currPref, value);
				}
			}
	
			return true;
		}
	};

	// ====================================================================================
	this.FoxClocks_PrefManager = new PrefManager();

}).call(this);